/************************************************************************
*                     SVR4 Driver-Kernel Interface			*
*************************************************************************
*									*
* These are the external definitions for SVR4 compatible DKI functions.	*
*									*
* Author:	David Grothe <dave@gcom.com>				*
*									*
* Copyright (C) 1997  David Grothe, Gcom, Inc <dave@gcom.com>		*
*									*
************************************************************************/
/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Cambridge,
 * MA 02139, USA.
 * 
 */

#ifndef	DKI_H
#define	DKI_H		1

#ident "@(#) LiS dki.h 2.4 3/27/01 19:55:18 "

#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif


#ifndef NULL
#define	NULL		( (void *) 0 )
#endif

/************************************************************************
*                           Timeouts                                    *
*************************************************************************
*									*
* The function timeout is alled with a pointer to a function to		*
* invoke at a later time, an argument to be passed to the function,	*
* and the number of ticks to wait.  'timeout' returns an integer	*
* that serves as a "handle" for the timer.  The untimeout function	*
* takes a timer handle as an argument and stops the timer.		*
*									*
* These two functions are defined in the dki.c file.  Their		*
* implementation depends upon lis_tmout() and lis_untimout() which	*
* are defined in either linux-mdep.c, which uses Linux timers to	*
* implement the functions, or in the port-mdep.c file, which implements	*
* a portable timer algorithm based on the Linux timer mechanism.	*
*									*
************************************************************************/

#ifdef __KERNEL__

typedef void	timo_fcn_t(caddr_t arg) ;
typedef int	toid_t ;			/* SVR4 */
typedef int	timeout_id_t ;			/* Solaris */

#define	timeout(fcn,arg,ticks)	lis_timeout_fcn(fcn,arg,ticks,__FILE__, __LINE__)
#define	untimeout		lis_untimeout

extern toid_t	lis_timeout_fcn(timo_fcn_t *timo_fcn, caddr_t arg, long ticks,
			    char *file_name, int line_nr) ;
extern toid_t	lis_untimeout(toid_t id) ;

extern void lis_terminate_dki(void);

#endif				/* __KERNEL__ */



#ifndef HZ
#define	HZ	100			/* ticks per second */
#endif

#if 0

The following is a sketch of a feature that would be nice to implement
someday.  The implementation problem with these routines is the amazing
messiness involved in performing the "mknod" and "unlink" functions
from within the kernel.

The Linux kernel does not have any nice routines that a driver can call
to easily perform these operations.  In the 2.2 kernels there used to
be routines "do_mknod" and "do_unlink" that were intende to be called
from kernel code to perform those functions.  However, "do_unlink" was
a static routine and therefore could not be used by outside driver code.
If you tried to supply the functionality of "do_unlink" in LiS the next
problem was that there were even more functions that were either static
or not exported.  So I gave up.

In the 2.4 kernel it is just as bad.  In this version the kernel guys
decided to *eliminate* altogether these "do_xxx" functions, leaving only
the system call functions with lots of dentry/iode manipulation code
written inline.  All of that must be duplicated in LiS just in order
to perform a simple "mknod" or "unlink" operation.

This is operating system architecture worthy of the best minds at Microsoft.

/************************************************************************
*                        Creating Nodes                                 *
*************************************************************************
*									*
* The following two routines can be used to create and remove nodes	*
* in the "/dev" directory.  These routines are used by drivers that	*
* wish to create these nodes dynamically at driver initialization time.	*
* They are especially useful for dynamically loaded drivers which obtain*
* their major device number at module load time.			*
*									*
************************************************************************/

/*
 * name		ASCII name of node to be created in "/dev" directory.
 * major_num	Major device number.
 * minor_num	Minor device number.
 * spec_type	S_IFCHR or S_IFREG for chr dev file or regular file, resp.
 * 		The above mnemonic are "or-ed" in with the traditional
 * 		mode bits.  Example: (0666 | I_IFCHR)
 * flag		CLONE_DEV or zero.
 */
extern int	lis_create_node(char *name,
			        int   major_num,
				int   minor_num, 
				int   spec_type,
				int   flag) ;
#define	CLONE_DEV	1

/*
 * name		ASCII name of node to remove in "/dev" directory.
 *
 * This is basically just an "unlink" call.
 */
extern int	lis_remove_node(char *name) ;

#endif



#endif				/* from top of file */
